!= $BJ|<MBPN.J?9U2r$N7w3&LL%l%Y%k$N7hDj!&@.AX7w$N9=B$$r7hDj(B
!
!= equilib tropopause
!
! Authors::   Masaki Ishiwatari
! Version::   $Id: moist_conv_adjust.f90,v 1.7 2011-12-02 04:06:36 yot Exp $ 
! Tag Name::  $Name: dcpam5-20120922 $
! Copyright:: Copyright (C) GFD Dennou Club, 2008. All rights reserved.
! License::   See COPYRIGHT[link:../../../COPYRIGHT]

module equilib_tropolvl_radeq
  !
  != $BJ?9U2r7w3&LL%l%Y%k$N7hDj(B
  !
  != tropopause level of equilibrium solution
  !
  ! <b>Note that Japanese and English are described in parallel.</b>
  !
  ! $BJ|<MBPN.J?9U2r$N7w3&LL%l%Y%k$N7hDj!&@.AX7w$N9=B$$r7hDj(B
  !
  ! In this module, tropopause level of equilibrium model is obtained.
  !
  !== Procedures List
  ! 
  ! EquilibTorpoLvlRadEq :: $B3%?'J|<M$N>l9g$N7w2rLL%l%Y%k$N7hDj(B
  ! ------------------ :: ------------
  ! EquilibTorpoLvlRadEq :: tropopause level
  !
  !== NAMELIST
  !
  ! NAMELIST#equilib_tropolvl_radeq_nml
  !

  ! $B%b%8%e!<%k0zMQ(B ; USE statements
  !

  ! $B3J;RE@@_Dj(B
  ! Grid points settings
  !
  use gridset, only: imax, & ! $B7PEY3J;RE@?t(B. 
                             ! Number of grid points in longitude
    &                jmax, & ! $B0^EY3J;RE@?t(B. 
                             ! Number of grid points in latitude
    &                kmax    ! $B1tD>AX?t(B. 
                             ! Number of vertical level

  ! $B<oJL7?%Q%i%a%?(B
  ! Kind type parameter
  !
  use dc_types, only: DP, &      ! $BG\@:EY<B?t7?(B. Double precision. 
    &                 STRING     ! $BJ8;zNs(B.       Strings. 

  ! NAMELIST $B%U%!%$%kF~NO$K4X$9$k%f!<%F%#%j%F%#(B
  ! Utilities for NAMELIST file input
  !
  use namelist_util, only: MaxNmlArySize
                              ! NAMELIST $B$+$iFI$_9~$`G[Ns$N:GBg%5%$%:(B. 
                              ! Maximum size of arrays loaded from NAMELIST

  ! $B%a%C%;!<%8=PNO(B
  ! Message output
  !
  use dc_message, only: MessageNotify


  ! $BJ|<MBPN.J?9U2r$N7w3&LL%l%Y%k$N7hDj(B
  ! stratospherice optical depth of equilibrium solution 
  !
  use equilib_strat_optdep, only: EquilibStratOptDep

  ! $B@k8@J8(B ; Declaration statements
  !
  implicit none
  private

  ! $B8x3+<jB3$-(B
  ! Public procedure
  !
  public:: EquilibTropoLvlRadEq
  public:: EquilibTropoLvlRadEqInit

  ! $B8x3+JQ?t(B
  ! Public variables
  !
  logical, save :: equilib_tropolvl_radeq_inited = .false.
                              ! $B=i4|@_Dj%U%i%0(B. 
                              ! Initialization flag

  ! $BHs8x3+JQ?t(B
  ! Private variables
  !
!  real(DP), save:: RH
!                              ! $BAjBP<>EY(B. 
!                              ! relative humidity
!  integer, save:: ItrtMax
!                              ! $B%$%F%l!<%7%g%s2s?t(B. 
!                              ! Number of iteration

  character(*), parameter:: module_name = 'equilib_tropolvl_radeq'
                              ! $B%b%8%e!<%k$NL>>N(B. 
                              ! Module name
  character(*), parameter:: version = &
    & '$Name: dcpam5-20120922 $' // &
    & '$Id: moist_conv_adjust.f90,v 1.7 2011-12-02 04:06:36 yot Exp $'
                              ! $B%b%8%e!<%k$N%P!<%8%g%s(B
                              ! Module version

contains

  !--------------------------------------------------------------------------------------

  subroutine EquilibTropoLvlRadEq( &
     &         xyz_Temp   , xyz_QVap, &
     &         xyr_Temp   ,  &
     &         xyr_Press  ,  &
     &         xyr_RadLUwFlux, xyr_RadLDwFlux &
     &        )
    !
    ! $BJ?9U2r$N7w3&LL%l%Y%k$r7hDj$9$k(B.
    ! $B3%?'J|<MJ?9U$r2>Dj$7$F@.AX7w$N29EY9=B$$r7hDj$9$k(B.
    !
    ! tropopause level
    ! stratospheric temperature

    ! $B%b%8%e!<%k0zMQ(B ; USE statements
    !

    ! $BJ*M}!&?t3XDj?t@_Dj(B
    ! Physical and mathematical constants settings
    !
    use constants0, only: &
      & StB                   ! $ \sigma_{SB} $ . 
                              ! $B%9%F%U%!%s%\%k%D%^%sDj?t(B. 
                              ! Stefan-Boltzmann constant

    ! $B;~9o4IM}(B
    ! Time control
    !
    use timeset, only: &
      & DelTime, &            ! $ \Delta t $
      & TimeN, &              ! $B%9%F%C%W(B $ t $ $B$N;~9o(B. Time of step $ t $. 
      & TimesetClockStart, TimesetClockStop

    ! $B%R%9%H%j%G!<%?=PNO(B
    ! History data output
    !
    use gtool_historyauto, only: HistoryAutoPut


    ! $B@k8@J8(B ; Declaration statements
    !
    implicit none

    real(DP), intent(inout):: xyz_Temp (0:imax-1, 1:jmax, 1:kmax)
                              ! $ T $ .     $B29EY(B. Temperature
    real(DP), intent(inout):: xyz_QVap (0:imax-1, 1:jmax, 1:kmax)
                              ! $ q $ .     $BHf<>(B. Specific humidity
    real(DP), intent(in):: xyr_Press (0:imax-1, 1:jmax, 0:kmax)
                              ! $ \hat{p} $ . $B5$05(B ($BH>@0?t%l%Y%k(B). 
                              ! Air pressure (half level)
    real(DP), intent(in):: xyr_RadLUwFlux (0:imax-1, 1:jmax, 0:kmax)
                              ! $BD9GH>e8~$-%U%i%C%/%9(B. 
                              ! Upward longwave flux
    real(DP), intent(in):: xyr_RadLDwFlux (0:imax-1, 1:jmax, 0:kmax)
                              ! $BD9GH2<8~$-%U%i%C%/%9(B. 
                              ! Downward longwave flux
    real(DP):: xyr_Temp (0:imax-1, 1:jmax, 0:kmax)
                              ! $ \hat{p} $ . $B5$05(B ($BH>@0?t%l%Y%k(B). 
                              ! Air pressure (half level)
    real(DP):: xyr_RadLFlux  ( 0:imax-1, 1:jmax, 0:kmax )   !" $B@5L#D9GH%U%i%C%/%9(B
    real(DP):: PIB    ( 0:imax-1, 1:jmax, 0:kmax ) !" $B&P#B!a&R#T(B**4
    real(DP):: OptDepH2OVap   ( 0:imax-1, 1:jmax, 0:kmax ) !" $B8w3XE*8|$5!'?e(B
    real(DP):: OptDepDryCom   ( 0:imax-1, 1:jmax, 0:kmax ) !" $B8w3XE*8|$5!'6u5$(B

    real(DP):: xy_PressTropopause ( 0:imax-1, 1:jmax )   !" $B7w3&LL05NO(B
    integer:: LevelTropopause ( 0:imax-1, 1:jmax )            !" $B7w3&LL$N9b$5(B
    integer:: LevelLFluxMax ( 0:imax-1, 1:jmax ) !" $B@5L#D9GH%U%i%C%/%9$,:GBg$H$J$k(Blevel

    real(DP):: ZZ
    real(DP):: ZZB

    integer::  i, j, k

    LevelLFluxMax  =  1

    ! $B@5L#J|<M$N7W;;(B
    xyr_RadLFlux     = xyr_RadLUwFlux     - xyr_RadLDwFlux

    ! $B@5L#>e8~$-D9GH%U%i%C%/%9$,2<$,$j;O$a$k%l%Y%k$r8+$D$1$k(B
    ! $B$3$s$J$KC1=c$@$H$&$^$/$$$+$J$$$_$?$$(B.
    do j = 1, jmax 
      do i = 0, imax-1
        do k = kmax,0,-1
            if( xyr_RadLFlux(i,j,k-1) .LT. xyr_RadLFlux(i,j,k) ) then
               LevelLFluxMax(i,j) = k
               exit
            end if
        end do
      end do
    end do

    LevelTropopause  = LevelLFluxMax

    ! $BHyD4@0(B
    do j = 1, jmax
      do i = 0, imax-1
 7777 continue

         ! $B8w3XE*?<$5$N7W;;(B
         CALL EquilibStratOptDep( &
     &         OptDepH2OVap, OptDepDryCom, &
     &         xyz_QVap, xyr_Press , &
     &         LevelTropopause )

         PIB ( i,j,LevelTropopause(i,j) ) = (  xyr_RadLUwFlux ( i,j,LevelTropopause(i,j) ) &
     &              - StB * xyr_Temp( i,j,LevelTropopause(i,j) )**4 ) &
     &                  * (1.5*OptDepH2OVap( i,j,LevelTropopause(i,j) ) + 1.0 )

         ZZ = PIB ( i,j,LevelTropopause(i,j) ) &
     &        - StB *  xyr_Temp( i,j,LevelTropopause(i,j) )**4

         IF( (ZZ .GT. 0.0d0) .AND. (LevelTropopause(i,j) .NE. 0) ) THEN
            LevelTropopause(i,j) = LevelTropopause(i,j) - 1
            ZZB = ZZ
            goto 7777
         END IF

         IF ( (ABS(ZZ) .GT. ABS(ZZB))  &
     &         .AND. (LevelTropopause(i,j) .LT. LevelLFluxMax(i,j)) ) THEN
            LevelTropopause (i,j) = LevelTropopause (i,j) + 1

            CALL EquilibStratOptDep( &
     &         OptDepH2OVap, OptDepDryCom, &
     &          xyz_QVap, xyr_Press , &
     &          LevelTropopause )
         end if
      end do
    end do

    ! $B3%?'J|<M$N<0$r;H$C$F@.AX7w$N>e8~$-%U%i%C%/%97hDj(B
    do j = 1, jmax
      do i = 0, imax-1
        do K = LevelTropopause(i,j), kmax
            xyr_RadLFlux( i,j,k ) = 2.0d0 * StB &
     &                  * ( xyr_Temp( i,j,LevelTropopause(i,j) ) )**4 &
     &                  /(1.5d0 * OptDepH2OVap( i,j,LevelTropopause(i,j) ) + 1.0d0)
        end do
      end do
    end do

    ! $B@.AX7w$N29EY9=B$(B
    do j = 1, jmax
      do i = 0, imax-1
        do K = LevelTropopause(i,j), kmax
            PIB ( i,j,K ) = xyr_RadLFlux ( i,j,K ) /2.0 &
     &                    * (  1.5*OptDepH2OVap( i,j,K ) + 1.0 )
        end do

        do K=LevelTropopause(i,j)+1, kmax
          xyz_Temp ( i,j,k ) = ( (PIB( i,j,k-1 )+PIB( i,j,k))/2.0d0 &
     &                          /StB )**0.25
            ! $BK\Ev$O@5$7$/Jd4V$9$Y$7(B
        end do

        do K=LevelTropopause(i,j)+1, kmax
            xyz_QVap ( i,j,k ) = xyz_QVap ( i,j,LevelTropopause ( i,j )+1 )
        end do
      end do
    end do

    do j = 1, jmax
      do i = 0, imax-1
        xy_PressTropopause(i,j)= xyr_Press(i,j,LevelTropopause(i,j))
      end do
    end do
    call HistoryAutoPut( TimeN, 'PressTropopause' , xy_PressTropopause)

  end subroutine EquilibTropoLvlRadEq

  !--------------------------------------------------------------------------------------

  subroutine EquilibTropoLvlRadEqInit
    !
    ! equilib_tropopause $B%b%8%e!<%k$N=i4|2=$r9T$$$^$9(B. 
    ! NAMELIST#equilib_tropopause_nml $B$NFI$_9~$_$O$3$N<jB3$-$G9T$o$l$^$9(B. 
    !
    ! "equilib_tropopause" module is initialized. 
    ! "NAMELIST#equilib_tropopause_nml" is loaded in this procedure. 
    !

    ! $B%b%8%e!<%k0zMQ(B ; USE statements
    !

    ! $BJ|<MBPN.J?9U2r$N7w3&LL%l%Y%k$N7hDj(B
    ! stratospherice optical depth of equilibrium solution 
    !
    use equilib_strat_optdep, only: EquilibStratOptDepInit


    ! NAMELIST $B%U%!%$%kF~NO$K4X$9$k%f!<%F%#%j%F%#(B
    ! Utilities for NAMELIST file input
    !
    use namelist_util, only: namelist_filename, NmlutilMsg, NmlutilAryValid

    ! $B%U%!%$%kF~=PNOJd=u(B
    ! File I/O support
    !
    use dc_iounit, only: FileOpen

    ! $B<oJL7?%Q%i%a%?(B
    ! Kind type parameter
    !
    use dc_types, only: STDOUT ! $BI8=`=PNO$NAuCVHV9f(B. Unit number of standard output

    ! $BJ8;zNsA`:n(B
    ! Character handling
    !
    use dc_string, only: StoA

    ! $B%R%9%H%j%G!<%?=PNO(B
    ! History data output
    !
    use gtool_historyauto, only: HistoryAutoAddVariable

    ! $B@k8@J8(B ; Declaration statements
    !
    implicit none

    integer:: unit_nml        ! NAMELIST $B%U%!%$%k%*!<%W%sMQAuCVHV9f(B. 
                              ! Unit number for NAMELIST file open
    integer:: iostat_nml      ! NAMELIST $BFI$_9~$_;~$N(B IOSTAT. 
                              ! IOSTAT of NAMELIST read



    ! NAMELIST $BJQ?t72(B
    ! NAMELIST group name
    !
!    namelist /equilib_tropopause_nml/ &
!      & RH, ItrtMax

          ! $B%G%U%)%k%HCM$K$D$$$F$O=i4|2=<jB3(B "moist_conv_adjust#CumAdjInit" 
          ! $B$N%=!<%9%3!<%I$r;2>H$N$3$H(B. 
          !
          ! Refer to source codes in the initialization procedure
          ! "moist_conv_adjust#MoistConvAdjustInit" for the default values. 
          !

    ! $B<B9TJ8(B ; Executable statement
    !

    if ( equilib_tropolvl_radeq_inited ) return

    call EquilibStratOptDepInit

    ! $B%G%U%)%k%HCM$N@_Dj(B
    ! Default values settings
!    XXXXXXXX  = 1.0d0

    !
!!$    FlagUse = .true.

    ! NAMELIST $B$NFI$_9~$_(B
    ! NAMELIST is input
    !
!    if ( trim(namelist_filename) /= '' ) then
!      call FileOpen( unit_nml, &          ! (out)
!        & namelist_filename, mode = 'r' ) ! (in)
!
!      rewind( unit_nml )
!      read( unit_nml,                  &  ! (in)
!        & nml = moist_adiabat_nml, &  ! (out)
!        & iostat = iostat_nml )           ! (out)
!      close( unit_nml )
!
!      call NmlutilMsg( iostat_nml, module_name ) ! (in)
!      call NmlutilMsg( iostat_nml, module_name ) ! (in)
!      call NmlutilMsg( iostat_nml, module_name ) ! (in)
!      call NmlutilMsg( iostat_nml, module_name ) ! (in)
!      if ( iostat_nml == 0 ) write( STDOUT, nml = moist_adiabat_nml )
!    end if

    ! $B%$%F%l!<%7%g%s2s?t(B, $BIT0BDj$N5vMF8m:9$N%A%'%C%/(B
    ! Check number of iteration, admissible error of unstability
    !
!    call NmlutilAryValid( module_name, & ! (in)
!      & AdjustCriterion, 'AdjustCriterion', &      ! (in)
!      & ItrtMax,    'ItrtMax' )          ! (in)


    ! $B%R%9%H%j%G!<%?=PNO$N$?$a$N$X$NJQ?tEPO?(B
    ! Register of variables for history data output
    !

    call HistoryAutoAddVariable( 'PressTropopause', &
      & (/ 'lon ', 'lat ',  'time' /), &
      & 'Tropopause Pressure', 'Pa' )

    ! $B0u;z(B ; Print
    !
    call MessageNotify( 'M', module_name, '----- Initialization Messages -----' )
!    call MessageNotify( 'M', module_name, '  RH               = %f', d = (/ RH /) )
!    call MessageNotify( 'M', module_name, '  ItrtMax              = %d', i = (/ ItrtMax /) )
!    call MessageNotify( 'M', module_name, '  AdjustCriterion      = (/ %*r /)', &
!      & r = real( AdjustCriterion(1:ItrtMax) ), n = (/ ItrtMax /) )
!    call MessageNotify( 'M', module_name, '' )
!    call MessageNotify( 'M', module_name, '-- version = %c', c1 = trim(version) )

    equilib_tropolvl_radeq_inited = .true.

  end subroutine EquilibTropoLvlRadEqInit

end module equilib_tropolvl_radeq
